<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2019 Amasty (https://www.amasty.com)
 * @package Amasty_Coupons
 *
 * FILOBLU - ADDED FUNCTIONS
 * @author - JORDY DE ROSA - jordy.derosa@filoblu.com
 */


namespace Amasty\Coupons\Helper;

use Magento\Framework\App\Area;
use Magento\Quote\Model\Quote\Address;
use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\SalesRule\Model\Coupon;
use Magento\SalesRule\Model\ResourceModel\Coupon as CouponResource;
use Amasty\Coupons\Model\CouponRenderer;
use Magento\Backend\Model\Session\Quote;
use Magento\Framework\App\State;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Model\StoreManagerInterface;
use Magento\SalesRule\Api\RuleRepositoryInterface;

/**
 * Class Data
 */
class Data
{
    /**
     * @var Coupon
     */
    private $couponModel;

    /**
     * @var CheckoutSession
     */
    private $session;

    /**
     * @var CouponResource
     */
    private $couponResource;

    /**
     * @var CouponRenderer
     */
    private $couponRenderer;

    /**
     * @var Quote
     */
    private $backendSession;

    /**
     * @var State
     */
    private $state;


    protected $storeManager;
    protected $rule;


    public function __construct(
        CheckoutSession $session,
        Coupon $couponModel,
        CouponResource $couponResourceModel,
        CouponRenderer $couponRenderer,
        Quote $backendSession,
        State $state,
        StoreManagerInterface $storeManager,
        RuleRepositoryInterface $rule
    ) {
        $this->couponModel = $couponModel;
        $this->session = $session;
        $this->couponResource = $couponResourceModel;
        $this->couponRenderer = $couponRenderer;
        $this->backendSession = $backendSession;
        $this->state = $state;
        $this->storeManager = $storeManager;
        $this->rule = $rule;
    }

    /**
     * @param bool $isRuleApplied
     * @param Address|null $address
     *
     * @return array|bool
     * @throws LocalizedException
     */
    public function getRealAppliedCodes($isRuleApplied = false, $address = null)
    {
        if ($address) {
            $quote = $address->getQuote();
        } else {
            $quote = $this->state->getAreaCode() === Area::AREA_ADMINHTML
                ? $this->backendSession->getQuote()
                : $this->session->getQuote();
        }

        if (!$quote->getCouponCode()) {
            return [];
        }

        $coupons = $this->couponRenderer->parseCoupon($quote->getCouponCode());
        $appliedRules = array_map('trim', explode(',', $quote->getAppliedRuleIds()));

        if (!$appliedRules) {
            return false;
        }

        foreach ($coupons as $key => $coupon) {
            $rule = $this->couponModel->loadByCode($coupon);
            if (!$this->couponResource->exists($coupon)
                || (!$isRuleApplied && !in_array($rule->getRuleId(), $appliedRules))
            ) {
                unset($coupons[$key]);
            }
        }

        $coupons = array_unique($coupons);

        return $coupons;
    }


    /**
     * @param $appliedCodes
     * @param $ruleId
     * @return array
     */
    public function countSameRuleCoupons($appliedCodes, $ruleId)
    {
        $sameRuleCounter = 0;
        $ruleUsedCodes = "";
        foreach ($appliedCodes as $code)
        {
            $rId = $this->couponModel->loadByCode($code)->getRuleId();
            if($rId == $ruleId)
            {
                $ruleUsedCodes .="($code)";
                $sameRuleCounter++;
            }
        }
        return ["count" => $sameRuleCounter,"coupons"=>$ruleUsedCodes];
    }


    /**
     * @param $ruleId
     * @return string
     */
    public function getRuleLabelById($ruleId)
    {
        $currentStoreId = $this->storeManager->getStore()->getId() ?? 0;
        //set discount label
        $storeLabels = $this->rule->getById($ruleId)->getStoreLabels() ?? '';
        $ruleDescription = [];
        if(is_array($storeLabels))
        {
            foreach ($storeLabels as $item)
            {
                $ruleDescription[$item->getStoreId()] = $item->getStoreLabel();
            }
            if(isset($ruleDescription[$currentStoreId]))
            {
                $descriptions = $ruleDescription[$currentStoreId];
            }
            else if(isset($ruleDescription[0]))
            {
                $descriptions = $ruleDescription[0];
            }
            else
            {
                $descriptions = '';
            }
        }

        return $descriptions;
    }


    /**
     * @param $simpleAction
     * @param $amount
     * @param $item
     * @param $totalQty
     * @return float|int
     */
    public function calculateDiscount($simpleAction,$amount,$item,$totalQty)
    {
        $discount = 0;
        $qty = $item->getQty();
        $price = $item->getPriceInclTax() * $qty;
        //not implemented actions : Buy X get Y Free and all automatically add product to cart actions
        switch ($simpleAction)
        {
            case "by_percent":
                $discount = $price*$amount/100;
                break;
            case "cart_fixed":
                $discount = $amount/$totalQty;
                break;
            case "by_fixed":
                $discount = $amount;
        }
        return $discount;
    }

}
